@@ -32,19 +32,21 @@
 #define __GEDIT_FILE_H__
 
 #include "gedit-mdi-child.h"
+#include "recent-files/egg-recent-view.h"
+#include "recent-files/egg-recent-item.h"
 
 void 		gedit_file_new 		(void);
 void 		gedit_file_open 	(GeditMDIChild *active_child);
 void 		gedit_file_close 	(GtkWidget *view);
-gboolean	gedit_file_save 	(GeditMDIChild *child);
+gboolean	gedit_file_save 	(GeditMDIChild *child, gboolean force);
 gboolean 	gedit_file_save_as 	(GeditMDIChild *child);
 void		gedit_file_save_all 	(void);
 gboolean	gedit_file_close_all 	(void);
 void		gedit_file_exit 	(void);
 gboolean	gedit_file_revert 	(GeditMDIChild *child);
 
-gboolean 	gedit_file_open_uri_list (GList* uri_list, gint line);
-gboolean 	gedit_file_open_recent   (GnomeRecentView *view, const gchar* uri, gpointer data);
+gboolean 	gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create);
+gboolean 	gedit_file_open_recent   (EggRecentView *view, EggRecentItem *item, gpointer data);
 gboolean 	gedit_file_open_single_uri (const gchar* uri);
 
 gboolean	gedit_file_open_from_stdin (GeditMDIChild *active_child);
